package com.sun.imageio.plugins.common;

import java.io.PrintStream;

public class LZWStringTable
{
    private static final int RES_CODES = 2;
    private static final short HASH_FREE = -1;
    private static final short NEXT_FIRST = -1;
    private static final int MAXBITS = 12;
    private static final int MAXSTR = 4096;
    private static final short HASHSIZE = 9973;
    private static final short HASHSTEP = 2039;
    byte[] strChr;
    short[] strNxt;
    short[] strHsh;
    short numStrings;
    int[] strLen;
    
    public LZWStringTable() {
        this.strChr = new byte[4096];
        this.strNxt = new short[4096];
        this.strLen = new int[4096];
        this.strHsh = new short[9973];
    }
    
    public int addCharString(short n, byte b) {
        if (this.numStrings >= 4096) {
            return 65535;
        }
        int hash;
        for (hash = hash(n, b); this.strHsh[hash] != -1; hash = (hash + 2039) % 9973) {}
        this.strHsh[hash] = this.numStrings;
        this.strChr[this.numStrings] = b;
        if (n == -1) {
            this.strNxt[this.numStrings] = -1;
            this.strLen[this.numStrings] = 1;
        }
        else {
            this.strNxt[this.numStrings] = n;
            this.strLen[this.numStrings] = this.strLen[n] + 1;
        }
        short numStrings = this.numStrings;
        this.numStrings = (short)(numStrings + 1);
        return numStrings;
    }
    
    public short findCharString(short n, byte b) {
        if (n == -1) {
            return (short)(b & 0xFF);
        }
        int hash = hash(n, b);
        int n2;
        while ((n2 = this.strHsh[hash]) != -1) {
            if (this.strNxt[n2] == n && this.strChr[n2] == b) {
                return (short)n2;
            }
            hash = (hash + 2039) % 9973;
        }
        return -1;
    }
    
    public void clearTable(int n) {
        this.numStrings = 0;
        for (int i = 0; i < 9973; ++i) {
            this.strHsh[i] = -1;
        }
        for (int n2 = (1 << n) + 2, j = 0; j < n2; ++j) {
            this.addCharString((short)(-1), (byte)j);
        }
    }
    
    public static int hash(short n, byte b) {
        return (((short)(b << 8) ^ n) & 0xFFFF) % 9973;
    }
    
    public int expandCode(byte[] array, int n, short n2, int n3) {
        if (n == -2 && n3 == 1) {
            n3 = 0;
        }
        if (n2 == -1 || n3 == this.strLen[n2]) {
            return 0;
        }
        int n4 = this.strLen[n2] - n3;
        int n5 = array.length - n;
        if (n5 > n4) {
        }
        else {
            n4 = n5;
        }
        int n6 = n4 - n4;
        for (int n7 = n + n4; n7 > n && n2 != -1; n2 = this.strNxt[n2]) {
            if (--n6 < 0) {
                array[--n7] = this.strChr[n2];
            }
        }
        if (n4 > n4) {
            return -n4;
        }
        return n4;
    }
    
    public void dump(PrintStream printStream) {
        for (int i = 258; i < this.numStrings; ++i) {
            printStream.println(" strNxt[" + i + "] = " + (int)this.strNxt[i] + " strChr " + Integer.toHexString(this.strChr[i] & 0xFF) + " strLen " + Integer.toHexString(this.strLen[i]));
        }
    }
}
